@@ -7,6 +7,7 @@ from unittest import mock
 import pytest
 import websocket
 
+from engineio import base_client
 from engineio import client
 from engineio import exceptions
 from engineio import json
@@ -197,7 +198,7 @@ class TestClient(unittest.TestCase):
 
     def test_disconnect_polling(self):
         c = client.Client()
-        client.connected_clients.append(c)
+        base_client.connected_clients.append(c)
         c.state = 'connected'
         c.current_transport = 'polling'
         c.queue = mock.MagicMock()
@@ -207,12 +208,12 @@ class TestClient(unittest.TestCase):
         c.disconnect()
         c.read_loop_task.join.assert_called_once_with()
         c.ws.mock.assert_not_called()
-        assert c not in client.connected_clients
+        assert c not in base_client.connected_clients
         c._trigger_event.assert_called_once_with('disconnect', run_async=False)
 
     def test_disconnect_websocket(self):
         c = client.Client()
-        client.connected_clients.append(c)
+        base_client.connected_clients.append(c)
         c.state = 'connected'
         c.current_transport = 'websocket'
         c.queue = mock.MagicMock()
@@ -222,12 +223,12 @@ class TestClient(unittest.TestCase):
         c.disconnect()
         c.read_loop_task.join.assert_called_once_with()
         c.ws.close.assert_called_once_with()
-        assert c not in client.connected_clients
+        assert c not in base_client.connected_clients
         c._trigger_event.assert_called_once_with('disconnect', run_async=False)
 
     def test_disconnect_polling_abort(self):
         c = client.Client()
-        client.connected_clients.append(c)
+        base_client.connected_clients.append(c)
         c.state = 'connected'
         c.current_transport = 'polling'
         c.queue = mock.MagicMock()
@@ -237,11 +238,11 @@ class TestClient(unittest.TestCase):
         c.queue.join.assert_not_called()
         c.read_loop_task.join.assert_not_called()
         c.ws.mock.assert_not_called()
-        assert c not in client.connected_clients
+        assert c not in base_client.connected_clients
 
     def test_disconnect_websocket_abort(self):
         c = client.Client()
-        client.connected_clients.append(c)
+        base_client.connected_clients.append(c)
         c.state = 'connected'
         c.current_transport = 'websocket'
         c.queue = mock.MagicMock()
@@ -251,7 +252,7 @@ class TestClient(unittest.TestCase):
         c.queue.join.assert_not_called()
         c.read_loop_task.join.assert_not_called()
         c.ws.mock.assert_not_called()
-        assert c not in client.connected_clients
+        assert c not in base_client.connected_clients
 
     def test_current_transport(self):
         c = client.Client()
@@ -388,7 +389,7 @@ class TestClient(unittest.TestCase):
         c._read_loop_websocket.assert_not_called()
         c._write_loop.assert_called_once_with()
         on_connect.assert_called_once_with()
-        assert c in client.connected_clients
+        assert c in base_client.connected_clients
         assert (
             c.base_url
             == 'http://foo/engine.io/?transport=polling&EIO=4&sid=123'
@@ -428,7 +429,7 @@ class TestClient(unittest.TestCase):
         c._read_loop_websocket.assert_not_called()
         c._write_loop.assert_called_once_with()
         on_connect.assert_called_once_with()
-        assert c in client.connected_clients
+        assert c in base_client.connected_clients
         assert (
             c.base_url
             == 'https://foo/engine.io/?transport=polling&EIO=4&sid=123'
@@ -497,7 +498,7 @@ class TestClient(unittest.TestCase):
             'http://foo', {}, 'engine.io'
         )
         on_connect.assert_called_once_with()
-        assert c in client.connected_clients
+        assert c in base_client.connected_clients
         assert (
             c.base_url
             == 'http://foo/engine.io/?transport=polling&EIO=4&sid=123'
@@ -540,7 +541,7 @@ class TestClient(unittest.TestCase):
         c._read_loop_websocket.assert_not_called()
         c._write_loop.assert_called_once_with()
         on_connect.assert_called_once_with()
-        assert c in client.connected_clients
+        assert c in base_client.connected_clients
 
     @mock.patch('engineio.client.time.time', return_value=123.456)
     @mock.patch(
@@ -561,6 +562,26 @@ class TestClient(unittest.TestCase):
             timeout=5
         )
 
+    @mock.patch('engineio.client.time.time', return_value=123.456)
+    @mock.patch(
+        'engineio.client.websocket.create_connection',
+        side_effect=[ConnectionError],
+    )
+    def test_websocket_connection_extra(self, create_connection, _time):
+        c = client.Client(websocket_extra_options={'header': {'Baz': 'Qux'},
+                                                   'timeout': 10})
+        with pytest.raises(exceptions.ConnectionError):
+            c.connect(
+                'http://foo', transports=['websocket'], headers={'Foo': 'Bar'}
+            )
+        create_connection.assert_called_once_with(
+            'ws://foo/engine.io/?transport=websocket&EIO=4&t=123.456',
+            header={'Foo': 'Bar', 'Baz': 'Qux'},
+            cookie=None,
+            enable_multithread=True,
+            timeout=10
+        )
+
     @mock.patch('engineio.client.time.time', return_value=123.456)
     @mock.patch(
         'engineio.client.websocket.create_connection',
@@ -634,7 +655,7 @@ class TestClient(unittest.TestCase):
         c._read_loop_websocket.assert_called_once_with()
         c._write_loop.assert_called_once_with()
         on_connect.assert_called_once_with()
-        assert c in client.connected_clients
+        assert c in base_client.connected_clients
         assert c.base_url == 'ws://foo/engine.io/?transport=websocket&EIO=4'
         assert c.sid == '123'
         assert c.ping_interval == 1
@@ -676,7 +697,7 @@ class TestClient(unittest.TestCase):
         c._read_loop_websocket.assert_called_once_with()
         c._write_loop.assert_called_once_with()
         on_connect.assert_called_once_with()
-        assert c in client.connected_clients
+        assert c in base_client.connected_clients
         assert c.base_url == 'wss://foo/engine.io/?transport=websocket&EIO=4'
         assert c.sid == '123'
         assert c.ping_interval == 1
@@ -858,7 +879,7 @@ class TestClient(unittest.TestCase):
         http.auth = None
         http.proxies = None
         http.cert = 'foo.crt'
-        c = client.Client(http_session=http)
+        c = client.Client(http_session=http, ssl_verify=False)
         c._read_loop_polling = mock.MagicMock()
         c._read_loop_websocket = mock.MagicMock()
         c._write_loop = mock.MagicMock()
@@ -868,7 +889,7 @@ class TestClient(unittest.TestCase):
 
         assert len(create_connection.call_args_list) == 1
         assert create_connection.call_args[1] == {
-            'sslopt': {'certfile': 'foo.crt'},
+            'sslopt': {'cert_reqs': ssl.CERT_NONE, 'certfile': 'foo.crt'},
             'header': {},
             'cookie': '',
             'enable_multithread': True,
@@ -1132,7 +1153,7 @@ class TestClient(unittest.TestCase):
         c._read_loop_websocket.assert_called_once_with()
         c._write_loop.assert_called_once_with()
         on_connect.assert_not_called()  # was called by polling
-        assert c not in client.connected_clients  # was added by polling
+        assert c not in base_client.connected_clients  # was added by polling
         assert c.base_url == 'http://foo'  # not changed
         assert c.sid == '123'  # not changed
         assert c.transport() == 'websocket'
@@ -1694,12 +1715,12 @@ class TestClient(unittest.TestCase):
         c._write_loop()
         assert c.state == 'connected'
 
-    @mock.patch('engineio.client.original_signal_handler')
+    @mock.patch('engineio.base_client.original_signal_handler')
     def test_signal_handler(self, original_handler):
         clients = [mock.MagicMock(), mock.MagicMock()]
-        client.connected_clients = clients[:]
-        client.connected_clients[0].is_asyncio_based.return_value = False
-        client.connected_clients[1].is_asyncio_based.return_value = True
-        client.signal_handler('sig', 'frame')
+        base_client.connected_clients = clients[:]
+        base_client.connected_clients[0].is_asyncio_based.return_value = False
+        base_client.connected_clients[1].is_asyncio_based.return_value = True
+        base_client.signal_handler('sig', 'frame')
         clients[0].disconnect.assert_called_once_with()
         clients[1].disconnect.assert_not_called()
